'''Base class for widgets'''
import copy

import pygame
from pygame.locals import *

class Widget(object):
    def __init__(self, cfg):
        self.name = ""
        self.cfg = cfg

        self.enabled = True
        #outline
        self.pos = [0,0]
        self.rect = [0,0,0,0]
        
        #Other
        self.label = None
        self.label_x = 0
        self.label_y = 0

        self.font_size = 16
        self.font = None
        self.myfont = None

        self.text_colours = {
        "default": [220,220,200],
        "on"     : [200,210,190],
        "off"    : [40,40,40]
        }

        self.text_colour = "default"

        self.set_font()
        self.text = ""
        #actions for all widgets
        self.click_action = []
        self.hover_action = []
        self.scroll_up_action = []
        self.scroll_down_action = []
        self.key_bind = None # Just a number.

        #Internal data and model data hook
        self.refs = { #Path is the full path up to the needed value-key, ref is the value-key.
                        'data':{'path':None,'ref':None,'copy':None}, 
                        'ena':{'path':None,'ref':None,'copy':None}, 
                        'dis':{'path':None, 'ref':None, 'copy':None},
                        'redraw':{'path':None,'ref':None,'copy':None}
                    }

        self.dirty = True
        self.hidden = False
        self.overlap = False
        self.layer = None
        self.force_data = False
        
        self.state = None
        
    def set_pos(self, pos):
        self.pos = pos
    
    def set_rel_pos(self, pos, rel):
        nr = [pos[0] - rel[0], pos[1] - rel[1]]
        new_pos = [self.pos[0] + nr[0], self.pos[1] + nr[1]]
        self.set_pos(new_pos)
        
    def set_rect(self, rect):
        self.rect = rect
        
    def shift_rect(self, shift):
        self.rect = [shift[0], shift[1], self.rect[2], self.rect[3]]
        self.dirty = True #Mark as dirty to make sure it gets rerendered.

    def get_pos_rect(self):
        return [self.pos[0], self.pos[1], self.rect[2], self.rect[3]]

    def on_click(self, m_pos=[0,0]):
        pass

    def on_hover(self, m_pos=[0,0]):
        pass

    def on_unhover(self):
        pass
        
    def set_state(self, state):
        pass

    def on_scroll_up(self, m_pos=[0,0]):
        return self.scroll_up_action
    
    def on_scroll_down(self, m_pos=[0,0]):
        return self.scroll_down_action
    
    def on_drag(self):
        return self

    def set_text(self, text):
        try:
            col = self.text_colours[self.text_colour]
        except:
            col = self.text_colour
        self.text = text
        self.label = self.myfont.render(str(self.text), 1, col)
        self.align_text()
        self.dirty = True

    def refresh_text(self):
        self.set_font()
        self.set_text(self.text)

    def refresh(self):
        self.refresh_text()
        
    def set_text_col(self, colour):
        self.text_col = colour
        self.refresh_text()
        
    def set_font(self):
        if self.font is not None:
            self.myfont = pygame.font.Font(self.font, self.font_size)
        else:
            self.myfont = pygame.font.Font(self.cfg['globals']['fonts']['standard'], self.font_size)
        self.dirty = True
        
    def align_text(self):
        if self.label:
            self.label_x = (self.rect[2] - self.label.get_width()) / 2
            self.label_y = (self.rect[3] - self.label.get_height()) / 2
            self.dirty = True

    def update(self):
        refs = self.refs
        for key in refs.keys():
            ref = refs[key]
            if ref['path'] != None:
                kd = ref['path'][ref['ref']]
                if self.force_data or kd != ref['copy']:
                    fn = "handle_" + key
                    if fn in dir(self):
                        exec("self." + fn + "(kd)")
                    else: #If the ref doesn't exist as a function, just pass it to the widget
                        self.handle_wild_ref(key, kd) #should delete bad refs somehow.
                    ref['copy'] = kd

    def handle_on(self, val):
        self.set_state(val)

    def handle_off(self, val):
        self.set_state(val, True)

    def handle_enable(self, val):
        self.enabled = val

    def handle_disable(self, val):
        self.enabled = not val

    def handle_data(self, val):
        self.set_text(val)

    def handle_redraw(self, val):
        self.dirty = True
        
    def handle_hide(self, val):
        self.hidden = val
        self.dirty = True
        
    def handle_unhide(self, val):
        self.hidden = not val
        self.dirty = True

    def handle_wild_ref(self, key, ref):
        pass
